package thesis.gui;

import java.awt.FileDialog;
import java.awt.Frame;
import java.io.File;

import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

public final class Application {
  
  private Application() {

  }
  
  public static void main(String[] args) {
    SwingUtilities.invokeLater(new Runnable() {
      @Override
      public void run() {
        String lafClassName = UIManager.getSystemLookAndFeelClassName();
        for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
          if ("Nimbus".equals(info.getName())) {
            lafClassName = info.getClassName();
            break;
          }
        }
        try {
          UIManager.setLookAndFeel(lafClassName);
        } catch (Exception ignored) {
          // keep default look & feel
        }
        
        new MainFrame().setVisible(true);
      }
    });
  }
  
  public static void showMessage(String message) {
    JOptionPane.showMessageDialog(null, message, "Poruka",
        JOptionPane.INFORMATION_MESSAGE);
  }
  
  public static void showError(String error) {
    JOptionPane.showMessageDialog(null, error, "Greška",
        JOptionPane.ERROR_MESSAGE);
  }

  public static boolean openFileDialog(int mode, JTextField textField) {
    FileDialog fd = new FileDialog((Frame)null, "", mode);
    fd.setDirectory(new File("").getAbsolutePath());
    fd.setLocationRelativeTo(null);
    fd.setVisible(true);
    
    if (fd.getFile() != null) {
      File file = new File(fd.getDirectory(), fd.getFile());
      textField.setText(file.getAbsolutePath());
      return true;
    } else {
      return false;
    }
  }

}
