package thesis.simulation;

import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import thesis.tree.Body;

public final class DynamicSimulator extends ThreadedSimulator {
  
  private int granularity;
  private AtomicInteger nextBodyIndex;
  
  private final class DynamicWorker extends Worker {
    
    @Override
    protected void work() throws InterruptedException {
      while (true) {
        int begin = nextBodyIndex.getAndAdd(granularity);
        int end = Math.min(begin + granularity, bodies.size());
        
        for (int i = begin; i < end; ++i) {
          integrator.calculateAcceleration(bodies.get(i), root);
        }
        
        if (end == bodies.size()) {
          break;
        }
      }
    }
    
  }
  
  public DynamicSimulator(List<Body> bodies, int steps, int numWorkers,
      float granularity) {
    
    super(bodies, steps, numWorkers);
    this.granularity = (int)Math.max(bodies.size() * granularity, 1);
    this.nextBodyIndex = new AtomicInteger();
  }
  
  @Override
  protected Worker createWorker(int index) {
    return new DynamicWorker();
  }
  
  @Override
  protected void doCalculateForces() throws InterruptedException {
    nextBodyIndex.set(0);
    super.doCalculateForces();
  }

}
