package thesis.simulation;

import java.util.List;
import java.util.concurrent.CyclicBarrier;
import java.util.concurrent.Semaphore;

import thesis.tree.Body;

public abstract class ThreadedSimulator extends Simulator {
  
  private Worker[] workers;
  private CyclicBarrier beginCalculation;
  private Semaphore endCalculation;
  
  protected abstract class Worker extends Thread {
    
    @Override
    public void run() {
      try {
        while (true) {
          beginCalculation.await();
          work();
          endCalculation.release();
        }
      } catch (Exception e) {
        // simulation ended or aborted - quit
      }
    }

    protected abstract void work() throws InterruptedException;
    
  }

  protected ThreadedSimulator(List<Body> bodies, int steps, int numWorkers) {
    super(bodies, steps);
    this.beginCalculation = new CyclicBarrier(numWorkers + 1);
    this.endCalculation = new Semaphore(0);
    
    workers = new Worker[numWorkers];
    for (int i = 0; i < numWorkers; ++i) {
      workers[i] = createWorker(i);
      workers[i].start();
    }
  }
  
  protected abstract Worker createWorker(int index);

  @Override
  protected void doCalculateForces() throws InterruptedException {
    try {
      beginCalculation.await();
      endCalculation.acquire(workers.length);
    } catch (Exception e) {
      throw new InterruptedException();
    }
  }
  
  @Override
  protected void cleanup() {
    super.cleanup();
    for (Worker worker : workers) {
      worker.interrupt();
    }
  }
  
  protected int getWorkersCount() {
    return workers.length;
  }
  
}
