package thesis.tree;

public class TreeNode {
  
  private final float size;
  private final Vector center;
  
  private float totalMass;
  private Vector centerOfMass;
  
  private TreeNode[] children;
  
  public TreeNode(Vector center, float size, Body body) {
    this(center, size, body.getMass(), body.getPosition());
  }
  
  private TreeNode(Vector center, float size, float mass, Vector position) {
    this.center = center;
    this.size = size;
    this.totalMass = mass;
    this.centerOfMass = position;
  }
  
  public void insert(Body body) {
    insert(body.getMass(), body.getPosition());
  }
  
  private void insert(float mass, Vector position) {
    if (children == null) {
      if (centerOfMass != null && centerOfMass.equals(position)) {
        totalMass += mass;
      } else {
        children = new TreeNode[4];
        passDown(totalMass, centerOfMass);
        totalMass = 0.0f;
        centerOfMass = null;
      }
    }
    if (children != null) {
      passDown(mass, position);
    }
  }

  private void passDown(float mass, Vector position) {
    Vector displacement = position.subtract(center);
    int index;
    if (displacement.getY() <= 0) {
      index = displacement.getX() <= 0 ? 0 : 1; 
    } else {
      index = displacement.getX() <= 0 ? 2 : 3;
    }
    
    if (children[index] == null) {
      float[] dx = { -1,  1, -1,  1};
      float[] dy = { -1, -1,  1,  1};
      Vector offset = new Vector(size / 2 * dx[index], size / 2 * dy[index]);
      children[index] =
          new TreeNode(center.add(offset), size / 2, mass, position); 
    } else {
      children[index].insert(mass, position);
    }
  }
  
  public void calculateCenterOfMass() {
    if (children != null) {
      centerOfMass = new Vector();
      for (TreeNode child : children) {
        if (child != null) {
          child.calculateCenterOfMass();
          totalMass += child.totalMass;
          centerOfMass = centerOfMass.add(
              child.centerOfMass.multiply(child.totalMass));
        }
      }
      centerOfMass = centerOfMass.divide(totalMass);
    }
  }
  
  public boolean isLeaf() {
    return children == null;
  }
  
  public float getSize() {
    return size;
  }
  
  public float getTotalMass() {
    return totalMass;
  }
  
  public Vector getCenterOfMass() {
    return centerOfMass;
  }
  
  public TreeNode[] getChildren() {
    return children;
  }
  
}
